<?php

// +---------------------------------------------+
// |     Copyright  2010 - 2018 InterPhoto       |
// |     http://www.weensoft.com                 |
// |     This file may not be redistributed.     |
// +---------------------------------------------+

include('includes/Admin.Core.php');

CheckAccess('managedatabase');


PrintHeader('Database');

@set_time_limit(0);

// ######################### Test that the backup directory exists and is writable ##########################

$backupDir = ADMINPATH. 'backup/';
$adminDirname = substr(str_replace(dirname(dirname(__FILE__)), '', dirname(__FILE__)), 1);
$backupUrl = BASEURL . $adminDirname . '/backup/';

$errors = array();

$backupEnabled = false;

if (!is_dir($backupDir))  $errors[] ='Database backup directory ('.$backupDir.') does not exist!';

@chmod($backupDir,0777);

if (!is_writable($backupDir))	$errors[] = 'Database backup directory ('.$backupDir.') is not writable! - CHMOD to 0777';

if (!empty($errors)){
	PrintErrors($errors, 'Configuration Error');
}else{
	$backupEnabled = true;
}

// ####################### FILE READ/WRITE USING GZIP  ######################

function openFileWrite($filename)
{
	if(function_exists('gzopen'))
	{
		$filename .= '.gz';
		$handle = gzopen($filename, "w9");
	}
	else
	{
		$handle = fopen($filename, "w");
	}
	return $handle;
}

function openFileRead($filename)
{
	if(function_exists('gzopen'))
	{
		$handle = gzopen($filename, "r");
	}
	else
	{
		$handle = fopen($filename, "r");
	}
	return $handle;
}

function writeFileData($handle, $data)
{
	if(function_exists('gzwrite'))
	{
		gzwrite($handle, $data);
	}
	else
	{
		fwrite($handle, $data);
	}
}


function readFileData($handle, $size)
{
	if(function_exists('gzread'))
	{
		$data = gzread($handle, $size);
	}
	else
	{
		$data = fread($handle, $size);
	}
	return $data;
}

function eof($handle)
{
	if(function_exists('gzeof'))
	{
		return gzeof($handle);
	}
	else
	{
		return feof($handle);
	}
}

function closeFile($handle)
{
	if(function_exists('gzclose'))
	{
		gzclose($handle);
	}
	else
	{
		fclose($handle);
	}
}



// ####################### END FILE READ FUNCTIONS ######################

function BackupTable($tablename, $fp)
{
	global $DB;

	if(isset($fp))
	{
		// Get the SQL to create the table
		$createTable = $DB->query_first("SHOW CREATE TABLE `$tablename`");

		// Drop if it exists
		$tableDump = "DROP TABLE IF EXISTS `$tablename`;\n" . $createTable['Create Table'] . ";\n\n";

		writeFileData($fp, $tableDump);

		// get data
		$getRows = $DB->query("SELECT * FROM `$tablename`");
		$fieldCount = $DB->get_num_fields();
		$rowCount = 0;

		while ($row = $DB->getrow($getRows))
		{
			$tableDump = "INSERT INTO `$tablename` VALUES(";

			$fieldcounter = -1;
			$firstfield = 1;

			// get each field's data
			while (++$fieldcounter < $fieldCount)
			{
				if (!$firstfield)
				{
					$tableDump .= ', ';
				}else{
					$firstfield = 0;
				}

				if (!isset($row["$fieldcounter"])){
					$tableDump .= 'NULL';
				}else if ($row["$fieldcounter"] != ''){
					$tableDump .= '\'' . addslashes($row["$fieldcounter"]) . '\'';
				}else	{
					$tableDump .= '\'\'';
				}
			}

			$tableDump .= ");\n";

			writeFileData($fp, $tableDump);
			$rowCount++;
		}

		writeFileData($fp, "\n\n\n");

		$msg = "Exported $rowCount rows from table '$tablename'.<br/>";
	}else	{
		$msg = "Failed to export table '$tablename'!<br/>";
	}

	return $msg;
}


function BackupSingleTable($tablename)
{
	global $backupDir, $backupUrl;

	$theverifycode = substr(md5(rand(0,9999)), 6, 12);

	$filename = $tablename . '_' . $theverifycode . '_' . date("ymd") . '.sql';
	$path = $backupDir . $filename;

	$fp = openFileWrite($path);

	if($fp)
	{
		$msg = BackupTable($tablename, $fp);
		closeFile($fp);
	}

	$msg .= '<br/>Database backup saved to this file: ' . $backupUrl . $filename . Iif(function_exists('gzopen'), '.gz'). '<br/>';

	return $msg;
}

function EmptyTable($tablename)
{
	global $DB;
	$DB->query("DELETE FROM `$tablename`");
	$msg .= '<br/>Table ' . $tablename . ' has been emptied.<br/>';

	return $msg;
}


function BatchBackupTable($tablenames)
{
	global $backupDir, $backupUrl;

	$theverifycode = substr(md5(rand(0,9999)), 6, 12);

	$filename = TABLE_PREFIX . $theverifycode . '_' . date("ymd") . '.sql';

	$path = $backupDir . $filename;

	$fp = openFileWrite($path);

	if($fp)
	{
		for($i = 0; $i < count($tablenames); $i++)
		{
			$msg = $msg . BackupTable($tablenames[$i], $fp);
		}
		closeFile($fp);
	}

	$msg .= '<br/>Database backup saved to this file: ' . $backupUrl . $filename . Iif(function_exists('gzopen'), '.gz'). '<br/>';

	return $msg;
}


function ParseQueries($sql, $delimiter)
{
    $matches = array();
    $output = array();

    $queries = explode($delimiter, $sql);
	$sql = "";

	$query_count = count($queries);
	for ($i = 0; $i < $query_count; $i++)
	{
		if (($i != ($query_count - 1)) || (strlen($queries[$i] > 0)))
		{
			$total_quotes = preg_match_all("/'/", $queries[$i], $matches);
			$escaped_quotes = preg_match_all("/(?<!\\\\)(\\\\\\\\)*\\\\'/", $queries[$i], $matches);
			$unescaped_quotes = $total_quotes - $escaped_quotes;

			if (($unescaped_quotes % 2) == 0)
			{
				$output[] = $queries[$i];
				$queries[$i] = "";
			}
			else
			{
				$temp = $queries[$i] . $delimiter;
				$queries[$i] = "";

				$complete_stmt = false;

				for ($j = $i + 1; (!$complete_stmt && ($j < $query_count)); $j++)
				{
					$total_quotes = preg_match_all("/'/", $queries[$j], $matches);
					$escaped_quotes = preg_match_all("/(?<!\\\\)(\\\\\\\\)*\\\\'/", $queries[$j], $matches);
					$unescaped_quotes = $total_quotes - $escaped_quotes;

					if (($unescaped_quotes % 2) == 1)
					{
						$output[] = $temp . $queries[$j];

						$queries[$j] = "";
						$temp = "";

						$complete_stmt = true;
						$i = $j;
					}
					else
					{
						$temp .= $queries[$j] . $delimiter;
						$queries[$j] = "";
					}
				}
			}
		}
	}

	return $output;
}


function RestoreBackup($filename)
{
	global $DB, $backupDir;

	$fp = openFileRead($backupDir . $filename);

	while (!eof($fp))
	{
		$query .= readFileData($fp, 10000);
	}

	closeFile($fp);

	$queries = ParseQueries($query, ';');

	for($i = 0; $i < count($queries); $i++)
	{
		$sql = trim($queries[$i]);

		if(!empty($sql))
		{
			$DB->query($sql);
		}
	}

	PrintRedirect('admin.database.php', 1);
}


function DeleteBackup($filename)
{
	global $backupDir;

	@unlink($backupDir . $filename);

	PrintRedirect('admin.database.php', 1);
}

// ####################### Display Action Results ######################

function PrintResults($title, $message)
{
	PrintSection($title);
	echo '<table width="100%" border="0" cellpadding="5" cellspacing="0">
	<tr>
	<td class="tdrow2" align="left">
	<font class="ohblueb">'.$title .' Results:</font><br/><br/>'. $message . '
	<br/></td>
	</tr>
	</table>';
	EndSection();
}

// ####################### Perform OP on Table ######################

function TableOperation($tablename, $OP)
{
	global $DB;

	$result = $DB->query_first("$OP TABLE `$tablename`");

	return " '" . $tablename . "' : <strong>" . $result['Msg_text'] . "</strong><br/>";
}


function BatchTableOperation($tablenames, $OP)
{
	global $DB;

	$msg = '';

	for($i = 0; $i < count($tablenames); $i++)
	{
		$msg = $msg . TableOperation($tablenames[$i], $OP);
	}

	return $msg;
}

// ######################### Instructions ##########################

function PrintInstructions()
{
	global $adminDirname;
	PrintSection('Instructions for Database Maintenance');
	echo '<table width="100%" border="0" cellpadding="5" cellspacing="0">
		<tr>
		<td class="tdrow2">
		You can use this tool to backup and maintain your InterPhoto database, the maintenance commands available are below:<br />
		<strong>Check</strong> - Checks the table for errors.<br />
		<strong>Optimize</strong> - Reclaims any wasted space.<br />
		<strong>Repair</strong> - Attempts to recover errors in the table.<br />
		<strong>Empty</strong> - Table "'.TABLE_PREFIX.'sessions" records user\'s login info, table "'.TABLE_PREFIX.'vvc" records Verification Codes. These two table can be emptied.<br /><br />
		Key Info:<br/> 
		1. You can backup one or all tables, all tables mean whole InterPhoto database;<br/>
		2. The backup files saved in the folder <b>./'.$adminDirname.'/backup/</b>;<br/>
		3. If you want backup all data of your site, also you need to download two folders <b>config/</b> and <b>MyWebsiteImages/</b>.
		</td>
		</tr>
		</table>';
	EndSection();
}

// ####################### List Backup Files #########################

function DisplayBackups()
{
	global $DB, $backupDir, $backupUrl, $adminDirname;

	PrintSection('Backup Files');

	echo '<table width="100%" border="0" cellpadding="5" cellspacing="0">
	<tr>
	<td class="tdrow1">Filename (./'.$adminDirname.'/backup/)</td>
	<td class="tdrow1">Filesize</td>
	<td class="tdrow1">Date</td>
	<td class="tdrow1" colspan="3" align="center">Operations</td></tr>';

	if (is_dir($backupDir))
	{
		$dir = opendir($backupDir);

		while (false !== ($file = readdir($dir)))
		{
			if(strpos(strtolower($file),'.sql') > 0)
			{
				$stats = stat($backupDir . $file);

				echo '<tr>
				<td class="tdrow3">' . $file . '</td>
				<td class="tdrow3">' . DisplayFilesize($stats['size']) . '</td>
				<td class="tdrow3">' . DisplayDate($stats['mtime']) . '</td>
				<td class="tdrow3"><a href="admin.database.php?dbaction=restorebackup&filename=' . $file . '" onclick="return confirm(\'Are you sure to restore this backup file?\n\nRe: All data entered after backup date will be removed.\');">Restore</a></td>
				<td class="tdrow3"><a href="' . $backupUrl . $file . '">Download</a></td>
				<td class="tdrow3"><a href="admin.database.php?dbaction=deletebackup&filename=' . $file . '" onclick="return confirm(\'Are you sure to delete this backup file?\');">Delete</a></td>
				</tr>';
			}
		}

	}
	echo '</table>';

	EndSection();
}

// ####################### List Database Tables ######################

function DisplayTables()
{
	global $DB, $backupEnabled;

	PrintInstructions();

	echo '<form method="post" action="admin.database.php" name="tables">
	<input type="hidden" name="dbaction" value=""/>';

	PrintSection('Database Tables');

	echo '<table width="100%" border="0" cellpadding="5" cellspacing="0">
	<tr>
	<td class="tdrow1">Select</td>
			<td class="tdrow1">Table Name</td>
			<td class="tdrow1">Rows</td>
			<td class="tdrow1">Data Length</td>
			<td class="tdrow1">Index Length</td>
			<td class="tdrow1">Wasted Space</td>
			<td class="tdrow1" colspan="5" align="center">Operations</td>
	</tr>';

	$gettables = $DB->query("SHOW TABLES FROM `" . $DB->database . "` LIKE '" . substr(TABLE_PREFIX, 0, -1) ."\_%'");

	while($table = $DB->getrow($gettables))
	{
			$tableinfo = $DB->query_first("SHOW TABLE STATUS LIKE '" . $table[0] . "'");

			echo '<tr>
			<td class="tdrow3"><input type="checkbox" name="tablenames[]" value="' . $tableinfo['Name'] . '" checkme="group" /></td>
			<td class="tdrow3">' . $tableinfo['Name'] . '</td>
			<td class="tdrow3">' . $tableinfo['Rows'] . '</td>
			<td class="tdrow3">' . DisplayFilesize($tableinfo['Data_length']) . '</td>
			<td class="tdrow3">' . DisplayFilesize($tableinfo['Index_length']) . '</td>
			<td class="tdrow3">' . Iif($tableinfo['Data_free'] > 0, '<b>', '') . DisplayFilesize($tableinfo['Data_free']) . Iif($tableinfo['Data_free'] > 0, '</b>', '') .  '</td>
			<td class="tdrow3"><a href="admin.database.php?dbaction=checktable&tablename=' . $tableinfo['Name'] . '">Check</a></td>
			<td class="tdrow3"><a href="admin.database.php?dbaction=optimizetable&tablename=' . $tableinfo['Name'] . '">Optimize</a></td>
			<td class="tdrow3"><a href="admin.database.php?dbaction=repairtable&tablename=' . $tableinfo['Name'] . '">Repair</a></td>
			<td class="tdrow3">' . Iif($backupEnabled, '<a href="admin.database.php?dbaction=backuptable&tablename=' . $tableinfo['Name'] . '">Backup</a>', 'Backup') . '</td>';
			
			if($tableinfo['Name'] == TABLE_PREFIX . 'sessions' OR $tableinfo['Name'] == TABLE_PREFIX . 'vvc')
			{
			   echo '<td class="tdrow3"><a href="admin.database.php?dbaction=emptytable&tablename=' . $tableinfo['Name'] . '" onclick="return confirm(\'Are you sure to empty this table?\');">Empty</a></td>';
			}
			else
			{
			   echo '<td class="tdrow3">-</td>';
			}
			
	echo '</tr>';
	}

	echo '<tr>
		<td class="tdrow1" colspan="6"><input type="checkbox" checkall="group" onclick="javascript: return select_deselectAll (\'tables\', this, \'group\');">&nbsp;Select All</td>
		<td class="tdrow1"><input type="submit" value="Check" onclick="document.forms[\'tables\'].dbaction.value = \'checkall\';"/></td>
		<td class="tdrow1"><input type="submit" value="Optimize" onclick="document.forms[\'tables\'].dbaction.value = \'optimizeall\';"/></td>
		<td class="tdrow1"><input type="submit" value="Repair" onclick="document.forms[\'tables\'].dbaction.value = \'repairall\';"/></td>
		<td class="tdrow1"><input type="submit" value="Backup" onclick="document.forms[\'tables\'].dbaction.value = \'backupall\';" ' . Iif($backupEnabled, '', 'disabled') . '/></td>
		<td class="tdrow1">-</td>
		</tr>
		</table>
		</form>';

	EndSection();

	DisplayBackups();

	PrintFooter();
}

// get the value of $action
$action = ForceIncomingString('dbaction');
$tablename = ForceIncomingString('tablename');
$filename = ForceIncomingString('filename');

switch ($action)
{
	case 'checktable':
		PrintResults('Check Table', TableOperation($tablename, 'CHECK'));
		break;
	case 'checkall':
		PrintResults('Check Table', BatchTableOperation($_POST['tablenames'], 'CHECK'));
		break;
	case 'optimizetable':
		PrintResults('Optimize Table', TableOperation($tablename, 'OPTIMIZE'));
		break;
	case 'optimizeall':
		PrintResults('Optimize Table',BatchTableOperation($_POST['tablenames'], 'OPTIMIZE'));
		break;
	case 'repairtable':
		PrintResults('Repair Table',TableOperation($tablename, 'REPAIR'));
		break;
	case 'repairall':
		PrintResults('Repair Table',BatchTableOperation($_POST['tablenames'], 'REPAIR'));
		break;
	case 'backuptable':
		PrintResults('Backup Table',BackupSingleTable($tablename));
		break;
	case 'backupall':
		PrintResults('Backup Table', BatchBackupTable($_POST['tablenames']));
		break;
	case 'emptytable':
		PrintResults('Empty Table', EmptyTable($tablename));
		break;
	case 'restorebackup':
		RestoreBackup($filename);
		break;
	case 'deletebackup':
		DeleteBackup($filename);
		break;
}

DisplayTables();

?>